// file ecash.c

#include "ecash.h"

#define NSF " NSF"
#define WRONG_IB_TYPE "WRONG IB TYPE"
#define NO_MONEY_FILE "NO MONEY FILE"
#define DIFFERENT_SYSTEM "DIFFERENT SYSTEM"
#define NO_COPROCESSOR "NO COPROCESSOR"

void setLCDMsgTemp(char* msg, int len)
{
   lcd_clear();
   lcd_putstring(msg, len);
   MsgBit = 1;
   MsgCount = 0;
}

void main(void)
{
#if PROCESSOR==DS89C420
   // enable internal 1k SRAM
   PMR |= 0x01;
   beep(1);
#ifdef CTM2
   // start the crystal multiplier
   PMR |= 0x10;
   // wait until xtal multiplier starts
   while(!(EXIF&0x08))
      ; // no-op
   // Activate the 2X clock mode
   PMR &= 0x1F;
#endif
#endif

   // initilialize timer0 for interrupt
   // must call before serial_init
   timer0Init();

   // initialize the serial port
   serial_init();
   //PRINTLN(FirmwareString);

   // initialize the LCD
   lcd_init();
   LCD_PRINT(FirmwareString);

   // initialize the onewire port
   ow_init();

   memset(BindData, 0xFF, 32);
   memset(BindCode, 0xFF, 7);
   memset(InitialSig, 0x00, 20);
   memset(SigningChlg, 0x00, 3);
   memset(COP_RomID, 0x00, 8);
   memset(COP_RomID_61S, 0x00, 8);

   prepCoprocessor();
   if(*COP_RomID==0 && *COP_RomID_61S==0)
   {
      PRINTLN(NO_COPROCESSOR);
      LCD_PRINT(NO_COPROCESSOR);
      for(;;);
   }

   softReset();

   OWOverdrive = 1;
   COPSelected = 0;
   for(;;)
   {
      // TODO: reset the watchdog timer

      // ensure we're looking at the customer port
      selectCustomer();

      // run the command processor
      commandProcessor();

      // check for timeout
      if(TimeOut)
      {
         //beep(3); // 3 beeps for timeout

         stopTimerSec();

         // if timeout occurred during update/or verification
         if(Stat_UpdateToken || Stat_Verify)
         {
            if(TokenTear)
            {
               // if last timeout was a tear, and it timed out again,
               // just forget about it.
               TokenTear = 0;
            }
            else
            {
               TokenTear = 1;
               // save a copy of the Rom ID
               memcpy(RomData2, Cust_RomID, 8);
               // save a copy of the New Record
               memcpy(ScratchRecord, NewRecord, 36);
            }
         }

         // Clear ecash state
         Status_Flags = 0;
         // wait for token to depart, before doing anything else
         Stat_WaitUntilGone = 1;
      }

      // Update the LCD message after a period of time, but only if
      // the token has departed
      if(MsgBit && !Stat_WaitUntilGone)
      {
         if(!(--MsgCount))
         {
            MsgBit = 0;

            if(Mode_LCDEnabled)
            {
               lcd_clear();
               lcd_putstring(LCDMessageDefault, 16);
            }
         }
      }

      // Poll for and read tokens if enabled.
      if(Mode_AutoPoll && !Status_Flags
         && !Ack_PollWaiting && !Ack_ReadWaiting
         && !Ack_DebitWaiting)
         Stat_Poll = 1;

      // If polling for a device
      if(Stat_Poll)
      {
         //COPR_STD_Bit=CUST_STD_Bit=COPR_OD_Bit=CUST_OD_Bit=0;
         //COPR_STD_Reset=CUST_STD_Reset=COPR_OD_Reset=CUST_OD_Reset=0;
         pollForToken();
         if(SuccessFlag)
         {
            Stat_Poll = 0;
            reportToken(1);

            if((Cust_RomID[0]==0x33 && COP_RomID_61S[0]==0) ||
               (Cust_RomID[0]==0x18 && COP_RomID[0]==0))
            {
               setLCDMsgTemp(DIFFERENT_SYSTEM, sizeof(DIFFERENT_SYSTEM)-1);
               errorCode = Err_NoValidToken;
               reportError();
               Stat_WaitUntilGone = 1;
            }
            else
            {
               startTimerSec();
               startTimerMS();

               // clear the lcd
               setLCDMsgTemp(" ", 1);

               if(TokenTear && (memcmp(Cust_RomID, RomData2, 8)==0))
               {
                  // This is a token that left and returned, skip to verify
                  Stat_Verify = 1;
                  memcpy(NewRecord, ScratchRecord, 36);
               }
               else if(Ack_PollEnabled)
               {
                  Ack_PollWaiting = 1;
                  TokenTear = 0;
               }
               else if(Mode_AutoRead)
               {
                  Stat_Read = 1;
                  TokenTear = 0;
               }
            }
         }
         else
         {
            if(errorCode==Err_TokenWrongType)
            {
               setLCDMsgTemp(WRONG_IB_TYPE, sizeof(WRONG_IB_TYPE)-1);
               reportError();
            }
         }
      }

      // If reading the device
      if(Stat_Read)
      {
         Stat_Read = 0;
         readToken();
         if(SuccessFlag)
         {
            startTimerSec();
            reportRead();
            if(Ack_ReadEnabled)
            {
               Ack_ReadWaiting = 1;
            }
            else if(Mode_AutoDebit)
            {
               Stat_DebitCert = 1;
            }
         }
         else
         {
            reportError();
            Stat_Poll = 1;
            //if(specificErrorCode!=Err_Spec_Bad_CRC_On_Cmd)
            {
               if(errorCode==Err_ReadAccount || errorCode==Err_Authenticate || errorCode==Err_Validate || errorCode==Err_GetChallenge)
               {
                  setLCDMsgTemp(DIFFERENT_SYSTEM, sizeof(DIFFERENT_SYSTEM)-1);
               }
               else
               {
                  setLCDMsgTemp(NO_MONEY_FILE, sizeof(NO_MONEY_FILE)-1);
               }
            }
         }
      }

      // debit and re-sign the certificate, fails on NSF
      if(Stat_DebitCert)
      {
         Stat_DebitCert = 0;
         debitCertificate();
         if(SuccessFlag)
         {
            //startTimerSec();
            Stat_UpdateToken = 1;
         }
         else
         {
            setLCDMsgTemp(BalString, balanceToString());
            lcd_putstring(NSF, sizeof(NSF)-1);

            reportError();
            Stat_WaitUntilGone = 1;
         }
      }

      // write the debited account to the token
      if(Stat_UpdateToken)
      {
         // retry this until it works
         writeAccountPage();
         if(SuccessFlag)
         {
            Stat_UpdateToken = 0;
            Stat_Verify = 1;
         }
         else
         {
            reportError();
         }
      }

      // verify the account data was written to token
      if(Stat_Verify)
      {
         // retry this until it works
         readBackVerify();
         if(SuccessFlag)
         {
            stopTimerMS();
            stopTimerSec();
            Stat_Verify = 0;
            TokenTear = 0;
            Stat_WaitUntilGone = 1;
            beep(1);

            // Report the balance
            reportDebit();
            if(Ack_DebitEnabled)
               Ack_DebitWaiting = 1;

            // Pulse the IO Bit
            if(Mode_Pulse)
            {
               switch(IOPinToPulse&0xF0)
               {
                  case 0x00:
                     P0 ^= (IOPinToPulse&0x0F);
                     msDelay(1);
                     P0 ^= (IOPinToPulse&0x0F);
                     break;
                  case 0x10:
                     P1 ^= (IOPinToPulse&0x0F);
                     msDelay(1);
                     P1 ^= (IOPinToPulse&0x0F);
                     break;
                  case 0x20:
                     P2 ^= (IOPinToPulse&0x0F);
                     msDelay(1);
                     P2 ^= (IOPinToPulse&0x0F);
                     break;
                  case 0x30:
                     P3 ^= (IOPinToPulse&0x0F);
                     msDelay(1);
                     P3 ^= (IOPinToPulse&0x0F);
                     break;
               }
            }

            // update the LCD
            if(Mode_LCDEnabled)
            {
               setLCDMsgTemp(BalString, balanceToString());
               if(!msTimerOverflow)
               {
                  lcd_putchar(' ');
                  lcd_putchar(binToHex(msCounter/100));
                  lcd_putchar(binToHex((msCounter%100)/10));
                  lcd_putchar(binToHex(msCounter%10));
                  lcd_putchar('m');
                  lcd_putchar('s');
               }
               else
               {
                  lcd_putchar(' ');
                  lcd_putchar('>');
                  lcd_putchar('2');
                  lcd_putchar('5');
                  lcd_putchar('5');
                  lcd_putchar('m');
                  lcd_putchar('s');
               }
            }
         }
         else
         {
            reportError();
            if(errorCode == Err_Tear_Unchanged)
            {
               // retry updating the token
               Stat_DebitCert = 1;
               Stat_Verify = 0;
            }
         }
      }

      // checking for Token departure
      if(Stat_WaitUntilGone)
      {
         stopTimerSec(); // no timeouts on wait

         selectCustomer();
         ow_readRomID();
         if(SuccessFlag)
         {
            if(memcmp(Cust_RomID, RomData, 8)==0)
            {
               waitUntilGoneCounter = 255;
               tick();
            }
            else
            {
               uchar test = memcmp(Cust_RomID, RomData, 8);
               // different token, report departure
               reportToken(0);
               Stat_WaitUntilGone = 0;
            }
         }
         else
         {
            if((--waitUntilGoneCounter)==0)
            {
               // gone for 100 resets, report departure
               reportToken(0);
               Stat_WaitUntilGone = 0;
            }
         }
      }
   }
}

void softReset()
{
   beep(2);

   //memset(LCDMessageDefault, ' ', 16);
   memcpy(LCDMessageDefault, " iButton  eCash ", 16);
   
   lcd_clear();
   LCD_PRINT(FirmwareString);
   MsgBit = 0;

   // Clear all the acknowledgment flags
   Ack_Flags = 0;

   // Clear ecash state
   Status_Flags = 0;

   // Set if we should poll for tokens
   Mode_AutoPoll = 1;
   // Set if we should read any token  shows up
   Mode_AutoRead = 1;
   // Set if we should debit any token that is authentic
   Mode_AutoDebit = 1;
   // enable overdrive mode by default
   Mode_ODEnabled = 1;
   // Enable the LCD by default
   Mode_LCDEnabled = 1;

   // initialize debit amount to 5 cents
   DebitAmount[0] = 1;
   DebitAmount[1] = 0;
   DebitAmount[2] = 0;

   // default second timeout value is 15 seconds
   TimeOutValue = 15;
   stopTimerSec();
   stopTimerMS();

   reportReset();
}






