// file ecinit.c
#include "ecash.h"

// -------------------------------------------------
// seg BITADDR
// -------------------------------------------------
bbyte SerialFlags;
DECLARE_BIT(SerialFlags, 0, SerialReady); // Set when inbound serial message is ready
DECLARE_BIT(SerialFlags, 1, SerialSend); // Set while sending serial message out

bbyte ECashFlags;
DECLARE_BIT(ECashFlags, 0, NeedAccess); // Set when we need to access the part anew
DECLARE_BIT(ECashFlags, 1, DS1963S); // Set when 1963S is customer iButton
DECLARE_BIT(ECashFlags, 2, DS1961S); // Set when 1961S is customer iButton
DECLARE_BIT(ECashFlags, 3, COPSelected); // Set when Co-Processor is selected
DECLARE_BIT(ECashFlags, 4, Copr61S); // Set when 61S coprocessor has been selected
DECLARE_BIT(ECashFlags, 5, OWOverdrive); // Set when in OverDrive mode
DECLARE_BIT(ECashFlags, 6, SuccessFlag); // Set when a function completes without error
DECLARE_BIT(ECashFlags, 7, TokenTear); // Set when Token departs before update/verification completes
//DECLARE_BIT(ECashFlags, 6, Mode_ReVal); // Set if re-value is enabled
//DECLARE_BIT(ECashFlags, 7, Verbose); // Set for verbose mode

bbyte Mode_Flags;
DECLARE_BIT(Mode_Flags, 0, Mode_AutoPoll); // Report any arival
DECLARE_BIT(Mode_Flags, 1, Mode_AutoRead); // Report value in token after arrival
DECLARE_BIT(Mode_Flags, 2, Mode_AutoDebit); // Debit token after authentication
DECLARE_BIT(Mode_Flags, 3, Mode_Pulse); // Make relay pulse on successful debit
DECLARE_BIT(Mode_Flags, 4, MODE_LCDEnabled); // Enable/Disable LCD
DECLARE_BIT(Mode_Flags, 5, Mode_ODEnabled); // Enable/Disable Overdrive communication

bbyte Ack_Flags;
DECLARE_BIT(Ack_Flags, 0, Ack_PollEnabled); // enable lock-step bit for polling
DECLARE_BIT(Ack_Flags, 1, Ack_PollWaiting); // indicates waiting for acknowledgment of poll
DECLARE_BIT(Ack_Flags, 2, Ack_ReadEnabled); // enable lock-step bit for reading
DECLARE_BIT(Ack_Flags, 3, Ack_ReadWaiting); // indicates waiting for acknowledgment of read
DECLARE_BIT(Ack_Flags, 4, Ack_DebitEnabled); // enable lock-step bit for debiting
DECLARE_BIT(Ack_Flags, 5, Ack_DebitWaiting); // indicates waiting for acknowledgment of debit

bbyte Status_Flags;
DECLARE_BIT(Status_Flags, 0, Stat_Poll); // Set when polling is in progress
DECLARE_BIT(Status_Flags, 1, Stat_Read); // Set when reading/authenticating token
DECLARE_BIT(Status_Flags, 2, Stat_DebitCert); // Set when debiting cert is in progress
DECLARE_BIT(Status_Flags, 3, Stat_UpdateToken); // Set when updating token is in progress
DECLARE_BIT(Status_Flags, 4, Stat_Verify); // Set when verify is in progress
DECLARE_BIT(Status_Flags, 5, Stat_WaitUntilGone); // Set when waiting for token departure

bbyte Timer_Flags;
DECLARE_BIT(Timer_Flags, 0, TimeOut); // Set when Seconds value has unflowed
DECLARE_BIT(Timer_Flags, 1, secTimerRun); // Set when TmrSeconds is counting down
DECLARE_BIT(Timer_Flags, 2, msTimerRun); // Set when msCounter is counting up
DECLARE_BIT(Timer_Flags, 3, msTimerOverflow); // Set when timer runs for more then 255 ms
DECLARE_BIT(Timer_Flags, 4, MsgBit); // Message Toggle Flag
DECLARE_BIT(Timer_Flags, 5, SecBit); // Toggles every Hz

bbyte crcl; // low-order byte of CRC, must be bit-addressable

// -------------------------------------------------
// seg DIRECT
// -------------------------------------------------

// Scratchpad memory store definitions -
dbyte crch; // High order byte of CRC accumulator
dbyte Retry; // 1-Wire Retry counter
dbyte DebitAmount[3]; // Debit Amount

// Stores for pages and secrets to be used in debiting -
dbyte COP_MAP; // Page used in COP to computer UDS
dbyte COP_UDS; // Unique Device Secret re-generated in COP
dbyte COP_UDP; // Page used to authenticate cust devices
dbyte COP_SDP; // Page used to sign new balances (MUST BE 0 or 8)
dbyte CUST_ECP; // Monetary Certificate Page

dbyte errorCode;

// temporary variables - intermediate values and counters
//dbyte temp1, temp2, temp3;

dbyte waitCounter; // used in wait2ms loop
dbyte SysTimer; // Timer post scaler (ticks down at 53ms)
dbyte TmrSeconds; // Counts down at 1Hz
dbyte TimeOutValue; // used to set the value for timeouts (in seconds)
dbyte msCounter; // counts up at 1kHz
dbyte QtrSeconds; // counts down each quarter second
dbyte milliseconds; // counts down millseconds

dbyte waitUntilGoneCounter; // Counts number of resets since token departed

dbyte IOPinToPulse; // Stored as portNum/BitMask (i.e. 31 is P3.0, 22 is P2.1)

dint Entropy; // entropy used for random transaction ID

// -------------------------------------------------
// seg XRAM
// -------------------------------------------------
xbyte LCDMessageDefault[16];
xbyte MsgCount; // Counts how long message is shown on LCD

xbyte COP_RomID_61S[8]; // Rom ID of co-processor iButton for DS1961S transactions
xbyte COP_RomID[8]; // Rom ID of co-processor iButton
xbyte Cust_RomID[8]; // Rom ID of Customer iButton

xbyte OldRecord[32+4];
xbyte NewRecord[32+4];
xbyte ScratchRecord[32+4];

xbyte MaxPrice[2]; // VMC Maximum Price
xbyte MinPrice[2]; // VMC Minimum Price (Keep in this order!)

xbyte CopySPMAC[20];

xbyte RomData[8];
xbyte RomData2[8];

xbyte BalString[11]; // Space for up to 10 characters plus EOL ("0.00------",0)

xbyte ErrString[2]; // Space for a string for an error code

xbyte BindData[34];
xbyte BindCode[7];
xbyte SigningChlg[3];
xbyte AuthSecret[47];
xbyte SignSecret[47];
xbyte InitialSig[20];
xbyte XSecret[8];
xbyte MoneyFileName[5]; // Monetary file name to seek in customer iButton

//ImageSize equ $-BindData
#define ImageSize  168

xbyte InitialSigLength;
xbyte ProviderNameLength;
xbyte ProviderNameBuffer[64];
xbyte AuxInfoLength;
xbyte AuxInfoBuffer[64];

// declare the certificate information
xdata StructCertificate Certificate;