// file ecsearch.c
#include "ecash.h"

xbyte LastDevice;
xbyte LastDiscrepancy;
xbyte SearchSerialNum[8];

void owResetSearch()
{
   LastDevice = 0;
   LastDiscrepancy = 0;
   SearchSerialNum[0] = 0;
}

void owNext()
{
   xbyte bit_test, search_direction, bit_number;
   xbyte last_zero, serial_byte_number;
   xbyte serial_byte_mask;

   // initialize for search
   bit_number = 1;
   last_zero = 0;
   serial_byte_number = 0;
   serial_byte_mask = 1;

   crcl = 0;

   // if the last call was not the last one
   SuccessFlag = 0;
   if (!LastDevice)
   {
      // reset the 1-wire
      ow_reset();
      // if there are no parts on 1-wire, return FALSE
      if (!SuccessFlag)
      {
         PRINTLN("No response to reset in search");
         // reset the search
         LastDevice = 0;
         LastDiscrepancy = 0;
         return;
      }

      // Issue the Search ROM command
      owBuffer = 0xF0; // the search command
      ow_touchbyte();

      // loop to do the search
      do
      {
         // read a bit...
         owBufferOut = 1;
         ow_touchbit();
         if(owBufferIn)
            bit_test = 0x02;
         else
            bit_test = 0x00;

         // ... and its compliment
         owBufferOut = 1;
         ow_touchbit();
         if(owBufferIn)
            bit_test |= 0x01;

         // check for no devices on 1-wire
         if (bit_test == 3)
            break;
         else
         {
            // all devices coupled have 0 or 1
            if (bit_test > 0)
            {
               if(bit_test&0x01)
                  search_direction = 0;
               else
                  search_direction = 1;
            }
            else
            {
               // if this discrepancy if before the Last Discrepancy
               // on a previous next then pick the same as last time
               if (bit_number < LastDiscrepancy)
               {
                  if(SearchSerialNum[serial_byte_number] & serial_byte_mask)
                     search_direction = 1;
                  else
                     search_direction = 0;
               }
               else
               {
                  // if equal to last pick 1, if not then pick 0
                  if(bit_number==LastDiscrepancy)
                     search_direction = 1;
                  else
                     search_direction = 0;
               }

               //DEBUGBLOCKLBL("search_direction = ", &search_direction, 1);

               // if 0 was picked then record its position in LastZero
               if (search_direction == 0)
                  last_zero = bit_number;

            }

            // set or clear the bit in the SerialNum byte serial_byte_number
            // with mask serial_byte_mask
            if (search_direction == 1)
              SearchSerialNum[serial_byte_number] |= serial_byte_mask;
            else
              SearchSerialNum[serial_byte_number] &= ~serial_byte_mask;

            // serial number search direction write bit
            owBufferOut = search_direction;
            ow_touchbit();

            // increment the byte counter bit_number
            // and shift the mask serial_byte_mask
            bit_number++;
            serial_byte_mask <<= 1;

            // if the mask is 0 then go to new SerialNum[portnum] byte serial_byte_number
            // and reset mask
            if (serial_byte_mask == 0)
            {
               crc8_byte(SearchSerialNum[serial_byte_number]);
               serial_byte_number++;
               serial_byte_mask = 1;
            }
         }
      }
      while(serial_byte_number < 8);  // loop until through all SerialNum[portnum] bytes 0-7

      SuccessFlag = 0;
      // if the search was successful then
      if (!((bit_number < 65) || crcl))
      {
         // search successful so set LastDiscrepancy[portnum],LastDevice[portnum],next_result
         LastDiscrepancy = last_zero;
         if(LastDiscrepancy==0)
            LastDevice = 1;
         SuccessFlag = 1;
      }
   }

   // if no device found then reset counters so next 'next' will be
   // like a first
   if (!SuccessFlag || !SearchSerialNum[0])
   {
      LastDiscrepancy = 0;
      LastDevice = 0;
      SuccessFlag = 0;
   }
}