// file ecserial.c
#include "ecash.h"
#include <string.h>

// size must be power of 2
xbyte SInBuf[SInBufSize];
dbyte SInBufPtr;

// SOutBuf is a ring buffer, size must be power of 2.
// LIFO, data goes in at the tail, leaves from head
xbyte SOutBuf[SOutBufSize];
dbyte SOutBufPtrHead, SOutBufPtrTail;

static void serial_interrupt (void) interrupt 4
{
   if(RI)
   {
      uchar serialChar = SBUF;
      RI = 0;
      if(!SerialReady)
      {
         if(serialChar==0x0D) // ASCII_CR
         {
            SerialReady = 1;
            SInBufPtr = 0;
         }
         else if(SInBufPtr<SInBufSize)
         {
            SInBuf[SInBufPtr] = serialChar;
            SInBufPtr++;
         }
      }
   }
   else if(TI)
   {
      TI = 0;
      if(SerialSend)
      {
         if(SOutBufPtrHead!=SOutBufPtrTail)
         {
            uchar serialChar = SOutBuf[SOutBufPtrHead];
            // crazy \r\n stuff for WinKit terminal (needs CRLN)
            /*if(serialChar=='\n')
            {
               SBUF = '\r';
               SOutBuf[SOutBufPtrHead] = '\r';
            }
            else if(serialChar=='\r')
            {
               SBUF = '\n';
               SOutBufPtrHead = (SOutBufPtrHead+1)&(SOutBufSize-1);
            }
            else*/
            {
               SBUF = serialChar;
               SOutBufPtrHead = (SOutBufPtrHead+1)&(SOutBufSize-1);
            }

         }
         else
         {
            SerialSend = 0;
         }
      }
   }
}

void serial_init(void)
{
   ES = 0; // stop serial interrupts
   TR1 = 0; // stop timer 1
   IT1 = 0; // Prevent Timer #1 interrupts
   IE1 = 0;

   // Timer/Counter #1 is used to generate the serial port baud rate clock.
   PCON |= 0x80; // baud rate double for serial0
   TMOD = (TMOD&0x0F) | 0x20; // timer 1 is an 8bit auto-reload counter

   // set 8 bit uart with variable baud from timer
   // enable receiver and clear RI and TI
   SCON = 0x50;
   TR1 = 1; // start timer 1

   TF1 = 0; // clear timer 1 overflow flag
   TL1 = TH1 = BAUD57600_RELOAD; // set reload value for timer

   // Init serial port vars
   SOutBufPtrHead = SOutBufPtrTail = 0;
   SInBufPtr = 0;
   SOutBuf[0] = 0;
   SerialReady = 0;
   SerialSend = 0;

   RI = 0; // clear any old data
   PS = 0; // set for low priority

   ES = 1; // start serial interrupts
   EA = 1; // make sure all interrupts are enabled
   REN = 1; // enable serial port reception
   //ET1 = 1; // start timer 1
   //IT1 = 1;
}

// kickstart the transmit, in case it hasn't already started
void serial_transmit()
{
   // check to see if there is a current transmission
   if(!SerialSend && SOutBufPtrHead!=SOutBufPtrTail)
   {
      // go ahead and send the first character to trigger interrupt
      // and get the ball rolling
      TI = 0;
      SBUF = SOutBuf[SOutBufPtrHead];
      SOutBufPtrHead = (SOutBufPtrHead+1)&(SOutBufSize-1);
      SerialSend = 1;
   }
}

void serial_putstring(uchar* msg, uchar len)
{
   do
   {
      putchar(*msg);
      msg++;
   }
   while(--len);
   /*
   uchar difference, roomLeft;

   // Check if the string is bigger than the buffer
   if(len>SOutBufSize)
   {
      // must truncate message
      len = SOutBufSize;
      msg[len-1] = '*'; // denote that we truncated it
   }

   // wait until there is enough room in the buffer
   do
   {
      difference = (SOutBufPtrHead - SOutBufPtrTail - 1)&(SOutBufSize - 1);
   }
   while(difference!=0 && len>difference);

   roomLeft = SOutBufSize-SOutBufPtrTail;
   if(len>roomLeft)
   {
      memcpy(&SOutBuf[SOutBufPtrTail], msg, roomLeft);
      memcpy(SOutBuf, &msg[roomLeft], len - roomLeft);
   }
   else
   {
      memcpy(&SOutBuf[SOutBufPtrTail], msg, len);
   }
   SOutBufPtrTail = (SOutBufPtrTail+len)&(SOutBufSize-1);

   // make sure it's transmitting
   serial_transmit();
   */
}

char putchar(char ch)
{
   dbyte serialTempVar = (SOutBufPtrTail+1)&(SOutBufSize-1);

   // wait until there is enough room in the buffer for one more char
   while(serialTempVar==SOutBufPtrHead)
      ;//no-op
   // put the character onto the buffer
   SOutBuf[SOutBufPtrTail] = ch;
   // increment the tail pointer
   SOutBufPtrTail = serialTempVar;

   // make sure it's transmitting
   serial_transmit();

   return ch;
}

void serial_puthexstring(uchar* msg, uchar len)
{
   do
   {
      putchar(binToHex(*msg>>4));
      putchar(binToHex(*msg));
      msg++;
   }
   while(--len);
}

