// file ecutil.c
#include "ecash.h"

// declare I/O bit for the beeper
DECLARE_BIT(P1, 0, beeperX);
DECLARE_BIT(P1, 1, beeperY);
DECLARE_BIT(P0, 0, LED);


void longToBytes(unsigned long longValue, uchar* buffer, uchar len)
{
   do
   {
      *buffer = longValue&0x0FF;
      longValue >>= 8;
      buffer++;
   }
   while(--len);
}

unsigned long bytesToLong(uchar* buffer, uchar len)
{
   unsigned long longValue = 0;
   do
   {
      longValue <<= 8;
      longValue |= buffer[len-1];
   }
   while(--len);

   return longValue;
}

uchar binToHex(uchar bits)
{
   bits &= 0x0F;
   if(bits<10)
      return '0' + bits;
   else
      return 'A' + (bits-10);
}

void byteToHex(uchar bits, uchar* hexValue)
{
   hexValue[1] = binToHex(bits);
   hexValue[0] = binToHex(bits>>4);
}

uchar hexToBin(uchar hexValue)
{
   if(hexValue>='0' && hexValue<='9')
   {
      return hexValue - '0';
   }
   else
   {
      return toupper(hexValue) - 'A' + 10;
   }
}

uchar hexToByte(uchar* hexValue)
{
   return (hexToBin(hexValue[0])<<4) | hexToBin(hexValue[1]);
}


// turns 3-byte balance into string
uchar balanceToString()
{
   xbyte buffer[8];
   xbyte cnt = 0;
   long longBalance;

   if(DS1961S)
   {
      if(Certificate.Record.rec33.FileLength==RECORD_A_LENGTH)
         longBalance = bytesToLong(Certificate.Record.rec33.Balance_A, 3);
      else
         longBalance = bytesToLong(Certificate.Record.rec33.Balance_B, 3);
   }
   else
   {
      longBalance = bytesToLong(Certificate.Record.rec.Balance, 3);
   }

   memset(buffer, '0', 8);

   while(longBalance>0)
   {
      buffer[7-cnt] = binToHex(longBalance%10);
      cnt++;
      longBalance /= 10;
   }

   if(cnt<3)
      cnt = 3;

   BalString[0] = '$';
   memcpy(&BalString[1], &buffer[8-cnt], cnt - 2);
   BalString[cnt-1] = '.';
   BalString[cnt] = buffer[6];
   BalString[cnt+1] = buffer[7];
   BalString[cnt+2] = 0;

   return cnt+2;
}

// subtracts debit amount from the provided 3-byte balance
void debitBalance(uchar* balance, uchar* newBalance)
{
//   long longBalance = (bytesToLong(balance, 3) - bytesToLong(DebitAmount, 3));
   long longBalance = bytesToLong(balance, 3);
   longBalance -= bytesToLong(DebitAmount, 3);
   if(longBalance<0)
   {
      DEBUGLN("      --- --- --->>> NSF!");
      errorCode = Err_DebitNSF;
      SuccessFlag = 0;
   }
   else
   {
      longToBytes(longBalance, newBalance, 3);
      SuccessFlag = 1;
   }
}

void beep(uchar count)
{
   uchar j, len = count;

   // make a beep
   do
   {
      j = 0;
      do
      {
         beeperY = 0;
         usDelay(200);
         beeperX = 0;
         usDelay(200);
         beeperY = 1;
         usDelay(200);
         beeperX = 1;
         usDelay(200);
      }
      while(--j);
      if(len>1)
         msDelay(200);
   }
   while(--len);
}

void tick()
{
   // make a tick
   uchar j = 15;
   do
   {
      beeperY = 0;
      usDelay(30);
      beeperX = 0;
      usDelay(30);
      beeperY = 1;
      usDelay(30);
      beeperX = 1;
      usDelay(30);
   }
   while(--j);
   msDelay(200);
}

void getAsciiHexBalance(uchar* buffer, uchar offset)
{
   if(DS1961S)
   {
      if(Certificate.Record.rec33.FileLength==RECORD_A_LENGTH)
      {
         byteToHex(Certificate.Record.rec33.Balance_A[0], &buffer[offset]);
         byteToHex(Certificate.Record.rec33.Balance_A[1], &buffer[offset+2]);
         byteToHex(Certificate.Record.rec33.Balance_A[2], &buffer[offset+4]);
      }
      else
      {
         byteToHex(Certificate.Record.rec33.Balance_B[0], &buffer[offset]);
         byteToHex(Certificate.Record.rec33.Balance_B[1], &buffer[offset+2]);
         byteToHex(Certificate.Record.rec33.Balance_B[2], &buffer[offset+4]);
      }
   }
   else
   {
      byteToHex(Certificate.Record.rec.Balance[0], &buffer[offset]);
      byteToHex(Certificate.Record.rec.Balance[1], &buffer[offset+2]);
      byteToHex(Certificate.Record.rec.Balance[2], &buffer[offset+4]);
   }
}