import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import com.dalsemi.onewire.adapter.*;
import com.dalsemi.onewire.container.*;
import com.dalsemi.onewire.application.sha.*;

/**
 * @author shughes
 *  
 */
public class eCashInit
   extends JTabbedPane
   implements ActionListener, Runnable
{
   private static final DSPortAdapter adapter = new USerialAdapter();

   private static final Font font = new Font("Courier New", Font.PLAIN, 12);
   private static final Font fontBig = new Font("Courier New", Font.BOLD, 13);

   private JButton coprInitButton = null, userInitButton = null;
   private JTextField coprSecret = null, userSecret = null,
            initialBalance = null;
   private JComboBox userTokenList = null;

   private static final byte[] initialSignature = new byte[]
      { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
   private static final byte[] signingChlg = new byte[]
      { 0, 0, 0 };
   private static final byte[] bindingData = new byte[]
      { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
       0, 0, 0, 0, 0, 0, 0, 0, 0 };
   private static final byte[] bindingCode = new byte[]
      { (byte) 0xFF, (byte) 0xFF, (byte) 0xFF, (byte) 0xFF, (byte) 0xFF,
       (byte) 0xFF, (byte) 0xFF };
   private static final byte serviceFileExt = 102;
   private static final byte[] serviceFilename = new byte[]
      { (byte) 'D', (byte) 'L', (byte) 'S', (byte) 'M' };
   private static final byte[] auxData = new byte[]
      { (byte) 'T', (byte) 'E', (byte) 'S', (byte) 'T' };
   private static final byte[] providerName = new byte[]
      { (byte) 'D', (byte) 'A', (byte) 'L', (byte) 'S', (byte) 'E',
       (byte) 'M', (byte) 'I' };

   public eCashInit()
   {
      String strBindingData = "binding data";
      System.arraycopy(strBindingData.getBytes(), 0, bindingData, 0,
         strBindingData.length());

      JPanel tmpPanel;
      JPanel coprInitPanel = new JPanel();
      coprInitPanel.setLayout(new BoxLayout(coprInitPanel, BoxLayout.Y_AXIS));
      coprInitPanel.add(getLabel("    "));
      coprInitPanel
               .add(getLabelBig("This Utility will initialize the Coprocessor DS1963S that "));
      coprInitPanel
               .add(getLabelBig("will be mounted on the eCash Evaluation board. "));
      coprInitPanel
               .add(getLabel("                                                   "));
      coprInitPanel
               .add(getLabel("1.  Remove a DS1963S from the bag and snap it into the "));
      coprInitPanel.add(getLabel("    blue-dot. "));
      coprInitPanel.add(getLabel("    "));
      coprInitPanel.add(getLabel("2.  Enter a secret. "));
      coprSecret = new JTextField("secret");
      coprSecret.setColumns(32);
      coprSecret.setFont(font);
      tmpPanel = getPanel(getLabel("          "));
      tmpPanel.add(coprSecret);
      coprInitPanel.add(tmpPanel);
      coprInitPanel
               .add(getLabel("    (Rememember this secret, it will be required to create user "));
      coprInitPanel.add(getLabel("    tokens.) "));
      coprInitPanel.add(getLabel("    "));
      coprInitPanel
               .add(getLabel("3.  Select the type of user token that will be used. "));
      userTokenList = new JComboBox();
      userTokenList.setForeground(Color.black);
      userTokenList.setBackground(Color.white);
      userTokenList.setFont(font);
      userTokenList.addItem("DS1963S");
      userTokenList.addItem("DS1961S");
      tmpPanel = getPanel(getLabel("          "));
      tmpPanel.add(userTokenList);
      coprInitPanel.add(tmpPanel);
      coprInitPanel.add(getLabel("    "));
      coprInitPanel.add(getLabel("4.  Click on this button when ready. "));
      coprInitButton = new JButton("Initialize Now");
      coprInitButton.setFont(fontBig);
      coprInitButton.addActionListener(this);
      tmpPanel = getPanel(getLabel("          "));
      tmpPanel.add(coprInitButton);
      coprInitPanel.add(tmpPanel);
      coprInitPanel.add(getLabel("    "));
      coprInitPanel
               .add(getLabel("5.  When complete, remove the DS1963S from the blue-dot "));
      coprInitPanel
               .add(getLabel("    and insert onto eCash evaluation board. "));
      coprInitPanel.add(getLabel("    "));
      coprInitPanel
               .add(getLabel("6.  Click on the 'User Token Initialization' tab above. "));
      coprInitPanel.add(getLabel("    "));
      this.add("Coprocessor Initialization", coprInitPanel);

      JPanel userInitPanel = new JPanel();
      userInitPanel.setLayout(new BoxLayout(userInitPanel, BoxLayout.Y_AXIS));
      userInitPanel.add(getLabel("    "));
      userInitPanel
               .add(getLabelBig("This Utility will initialize the User Tokens (DS1963S or DS1961S) "));
      userInitPanel
               .add(getLabelBig("that will be demonstrated with the eCash Evaluation board. "));
      userInitPanel
               .add(getLabel("                                                   "));
      userInitPanel
               .add(getLabel("1.  Remove a DS1963S or DS1961S from the bag and snap it "));
      userInitPanel.add(getLabel("    into the blue-dot. "));
      userInitPanel.add(getLabel("    "));
      userInitPanel.add(getLabel("2.  Enter a secret. "));
      userSecret = new JTextField("secret");
      userSecret.setColumns(32);
      userSecret.setFont(font);
      tmpPanel = getPanel(getLabel("          "));
      tmpPanel.add(userSecret);
      userInitPanel.add(tmpPanel);
      userInitPanel
               .add(getLabel("    (This secret must be the same one used to initialize the "));
      userInitPanel.add(getLabel("    coprocessor.) "));
      userInitPanel.add(getLabel("    "));
      userInitPanel
               .add(getLabel("3.  Enter the starting balance (in dollars). "));
      initialBalance = new JTextField("500");
      initialBalance.setColumns(32);
      initialBalance.setFont(font);
      tmpPanel = getPanel(getLabel("          "));
      tmpPanel.add(initialBalance);
      userInitPanel.add(tmpPanel);
      userInitPanel.add(getLabel("    "));
      userInitPanel.add(getLabel("4.  Click on this button when ready. "));
      userInitButton = new JButton("Initialize Now");
      userInitButton.setFont(fontBig);
      userInitButton.addActionListener(this);
      tmpPanel = getPanel(getLabel("          "));
      tmpPanel.add(userInitButton);
      userInitPanel.add(tmpPanel);
      userInitPanel.add(getLabel("    "));
      userInitPanel
               .add(getLabel("5.  When complete, remove the iButton from the blue-dot.  To "));
      userInitPanel
               .add(getLabel("    initialize another user token, start over from step 1. "));
      userInitPanel.add(getLabel("    "));
      userInitPanel.add(getLabel("    "));
      this.add("User Token Initialization", userInitPanel);
   }

   public void setEnableAll(boolean enabled)
   {
      this.setEnabled(enabled);
      coprSecret.setEnabled(enabled);
      userTokenList.setEnabled(enabled);
      coprInitButton.setEnabled(enabled);
      userSecret.setEnabled(enabled);
      initialBalance.setEnabled(enabled);
      userInitButton.setEnabled(enabled);
   }

   public void actionPerformed(ActionEvent ae)
   {
      actionEventSource = ae.getSource();
      if ( actionEventSource == coprInitButton
           || actionEventSource == userInitButton )
      {
         this.setEnableAll(false);
         new Thread(this).start();
      }
   }

   // All buttons are disabled for each action event,
   // so this should be a safe parameter.
   Object actionEventSource = null;

   public void run()
   {
      if ( actionEventSource == coprInitButton )
      {
         userSecret.setText(coprSecret.getText());

         OneWireContainer owc = null;
         OneWireContainer18 owc18 = null;
         try
         {
            adapter.beginExclusive(true);
            adapter.setSpeed(DSPortAdapter.SPEED_REGULAR);
            adapter.targetFamily(0x18);
            owc = adapter.getFirstDeviceContainer();
            while ( owc != null && owc18 == null )
            {
               if ( owc instanceof OneWireContainer18 )
                  owc18 = (OneWireContainer18) owc;
               else
                  owc = adapter.getNextDeviceContainer();
            }
         }
         catch ( Exception e )
         {
            e.printStackTrace();
         }
         finally
         {
            adapter.endExclusive();
         }

         if ( owc18 == null )
         {
            JOptionPane.showMessageDialog(eCashInit.this,
               getLabel("No DS1963S was found on the 1-Wire Network"),
               "An Error Occurred", JOptionPane.ERROR_MESSAGE);
            setEnableAll(true);
            return;
         }

         byte[] signSecret = ("signing " + coprSecret.getText()).getBytes();
         byte[] authSecret = ("authentication " + coprSecret.getText())
                  .getBytes();
         String coprFilename = "COPR.";
         if ( userTokenList.getSelectedItem().equals("DS1963S") )
         {
            coprFilename += 0x18;
         }
         else
         {
            coprFilename += 0x33;
            authSecret = SHAiButtonCopr.reformatFor1961S(authSecret);
         }

         try
         {
            adapter.beginExclusive(true);
            SHAiButtonCopr copr = new SHAiButtonCopr(owc18, coprFilename,
                     true, 8, 7, 9, 1, 1, serviceFileExt, serviceFilename,
                     providerName, bindingData, bindingCode, auxData,
                     initialSignature, signingChlg, signSecret, authSecret);
         }
         catch ( Exception e )
         {
            JOptionPane.showMessageDialog(eCashInit.this,
               getLabel("Exception Occurred: " + e.getMessage()),
               "An Error Occurred", JOptionPane.ERROR_MESSAGE);
            setEnableAll(true);
            return;
         }
         finally
         {
            adapter.endExclusive();
         }

         JOptionPane.showMessageDialog(eCashInit.this,
            getLabel("Coprocessor was successfully initialized."), "Success",
            JOptionPane.PLAIN_MESSAGE);
         setEnableAll(true);
         return;
      }
      else if ( actionEventSource == userInitButton )
      {
         OneWireContainer owc = null;
         OneWireContainer33 owc33 = null;
         OneWireContainer18 owc18 = null;
         try
         {
            adapter.beginExclusive(true);
            adapter.setSpeed(DSPortAdapter.SPEED_REGULAR);
            adapter.targetFamily(new byte[]
               { (byte) 0x18, (byte) 0x33, (byte) 0xB3 });
            owc = adapter.getFirstDeviceContainer();
            while ( owc != null && (owc18 == null && owc33 == null) )
            {
               if ( owc instanceof OneWireContainer18 )
                  owc18 = (OneWireContainer18) owc;
               else if ( owc instanceof OneWireContainer33 )
                  owc33 = (OneWireContainer33) owc;
               else
                  owc = adapter.getNextDeviceContainer();
            }

         }
         catch ( Exception e )
         {
         }
         finally
         {
            adapter.endExclusive();
         }
         if ( owc18 == null && owc33 == null )
         {
            JOptionPane
                     .showMessageDialog(
                        eCashInit.this,
                        getLabel("No DS1963S or DS1961S was found on the 1-Wire Network"),
                        "An Error Occurred", JOptionPane.ERROR_MESSAGE);
            setEnableAll(true);
            return;
         }

         byte[] signSecret = ("signing " + userSecret.getText()).getBytes();
         byte[] authSecret = ("authentication " + userSecret.getText())
                  .getBytes();
         if ( owc33 != null )
         {
            authSecret = SHAiButtonCopr.reformatFor1961S(authSecret);
         }

         try
         {
            adapter.beginExclusive(true);
            SHAiButtonUser sha = null;
            SHATransaction trans = null;
            SHAiButtonCoprVM copr = new SHAiButtonCoprVM(new byte[8], 8, 7,
                     9, 1, 1, serviceFileExt, serviceFilename, providerName,
                     bindingData, bindingCode, auxData, initialSignature,
                     signingChlg, signSecret, authSecret);
            if ( owc18 != null )
            {
               sha = new SHAiButtonUser18(copr, owc18, true, authSecret);
               trans = new SHADebit(copr, Integer.parseInt(initialBalance
                        .getText()) * 100, 50);
            }
            else if ( owc33 != null )
            {
               sha = new SHAiButtonUser33(copr, owc33, true, authSecret);
               trans = new SHADebitUnsigned(copr, Integer
                        .parseInt(initialBalance.getText()) * 100, 50);
            }
            if ( !trans.setupTransactionData(sha) )
            {
               JOptionPane.showMessageDialog(eCashInit.this,
                  getLabel("Failed to setup transaction data ("
                           + trans.getLastError() + ","
                           + trans.getLastCoprError() + ")"),
                  "An Error Occurred", JOptionPane.ERROR_MESSAGE);
               setEnableAll(true);
               return;
            }
         }
         catch ( Exception e )
         {
            JOptionPane.showMessageDialog(eCashInit.this,
               getLabel("Exception Occurred: " + e.getMessage()),
               "An Error Occurred", JOptionPane.ERROR_MESSAGE);
            setEnableAll(true);
            return;
         }
         finally
         {
            adapter.endExclusive();
         }

         JOptionPane.showMessageDialog(eCashInit.this,
            getLabel("User Token was successfully initialized."), "Success",
            JOptionPane.PLAIN_MESSAGE);
         setEnableAll(true);
         return;
      }
      else
      {
         setEnableAll(true);
         return;
      }
   }

   public static void main(String[] args)
   {
      eCashInit ecinit = new eCashInit();
      ecinit.setEnableAll(false);

      JFrame mainFrame = new JFrame("eCash Initialization Utility");
      mainFrame.getContentPane().add(ecinit, BorderLayout.CENTER);
      mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

      JPanel panel;
      JPanel portSelectPanel = new JPanel();
      portSelectPanel.setLayout(new BoxLayout(portSelectPanel,
               BoxLayout.Y_AXIS));
      portSelectPanel
               .add(getLabelBig("Welcome to the eCash iButton Initialization Utility"));
      portSelectPanel
               .add(getLabel("                                                   "));
      portSelectPanel
               .add(getLabel("1.  Plug in the DS9097U-S09 (serial adapter) into the "));
      portSelectPanel.add(getLabel("    serial COM port of the PC"));
      portSelectPanel
               .add(getLabel("                                                   "));
      portSelectPanel
               .add(getLabel("2.  Plug the DS1402D-DR8 (blue-dot) into the adapter."));
      portSelectPanel
               .add(getLabel("                                                   "));
      portSelectPanel
               .add(getLabel("3.  Remove any iButtons plugged into the blue-dot."));
      portSelectPanel
               .add(getLabel("                                                   "));
      portSelectPanel.add(getLabel("4.  Select the PC serial COM port "));

      JComboBox portList = new JComboBox();
      portList.setFont(fontBig);
      portList.setForeground(Color.black);
      portList.setBackground(Color.white);

      Enumeration ports = adapter.getPortNames();
      while ( ports.hasMoreElements() )
      {
         portList.addItem(ports.nextElement());
      }

      JPanel tmpPanel = (JPanel) getLabel("                    ");
      tmpPanel.add(portList);
      portSelectPanel.add(tmpPanel);
      portSelectPanel
               .add(getLabel("                                                   "));

      Dimension dScreen = Toolkit.getDefaultToolkit().getScreenSize();
      mainFrame.pack();
      Dimension dFrame = mainFrame.getSize();
      mainFrame.setLocation((dScreen.width - dFrame.width) / 2,
         (dScreen.height - dFrame.height) / 2);
      mainFrame.setVisible(true);

      boolean adapterDetected = false;
      while ( !adapterDetected )
      {
         int result = JOptionPane.showConfirmDialog(mainFrame,
            portSelectPanel, "eCash iButton Initialization Utility",
            JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE);

         if ( result == JOptionPane.OK_OPTION )
         {
            try
            {
               
               adapter.selectPort(portList.getSelectedItem().toString());
               adapterDetected = adapter.adapterDetected();
            }
            catch ( Exception e )
            {
               e.printStackTrace();
            }

            if ( !adapterDetected )
            {
               JOptionPane.showMessageDialog(mainFrame,
                  getLabelBig("No Adapter Detected on Port: "
                              + portList.getSelectedItem()),
                  "No Adapter Detected", JOptionPane.ERROR_MESSAGE);
            }
            else
            {
               ecinit.setEnableAll(true);
               mainFrame.pack();
            }
         }
         else
         {
            mainFrame.dispose();
            System.exit(0);
         }
      }
   }

   private static JComponent getLabelBig(String msg)
   {
      JLabel lbl = new JLabel(msg);
      lbl.setFont(fontBig);
      lbl.setForeground(Color.black);
      return getPanel(lbl);
   }

   private static JComponent getLabel(String msg)
   {
      JLabel lbl = new JLabel(msg);
      lbl.setFont(font);
      lbl.setForeground(Color.black);
      return getPanel(lbl);
   }

   private static JPanel getPanel(JComponent comp)
   {
      JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 20, 0));
      panel.add(comp);
      return panel;
   }
}
